/***************************************************************************/
/*Defining the MACRO to discern between single sensor and dual sensor mode */
/***************************************************************************/
#ifdef ECG_MODE

	#define FLAG_ECG 1
	#define FLAG_ACC 0

#endif

#ifdef ACC_MODE

	#define FLAG_ECG 0
	#define FLAG_ACC 1

#endif

#ifdef BLENDED_MODE

	#define FLAG_ECG 1
	#define FLAG_ACC 1

#endif




void blending_wrapper(int index, double *output_array)
{	
	static int flag_for_over=0;					/*Flag indicating if the secure window was activated */
	int chandata_qt[lines];				/*Array of ecg signal */
	int chandata_acc_x[lines];			/*Array of accelerometer signal along the first axes */
	int chandata_acc_y[lines];			/*Array of accelerometer signal along the second axes */
	int chandata_acc_z[lines];			/*Array of accelerometer signal along the third axes */
	FILE *fp;
	int i=0;
	double RR_QT;							/*RR predicted by the QT sensor measured in seconds*/
	double RR_ACC;							/*RR predicted by the Accelerometer sensor measured in seconds*/
	double RR_BLEND;						/*RR predicted by the blended sensor measured in seconds*/
    	double RR;
	inizialization_qt();				/*Initilising the global array of the QT values*/
	count_flag=0;
	
	/*printf("...............................");*/
                if(FLAG_FOR_DEBUG)
		{	
			fp=fopen("debugging.txt","a+"); if(fp==NULL) THROW_ERROR
			fprintf(fp,"\n\n Inside blending wrapper, iteration num=%d \n\n\n",index);
			fclose(fp); 
		}
		inizialization_rr(); 				/*Initialising the Global array for the R peak locations	*/
		if(FLAG_FOR_DEBUG)
		{
			fp=fopen("debugging.txt","a+"); if(fp==NULL) THROW_ERROR
			fprintf(fp,"RR intervals global initialized \n");
			fclose(fp); 
		}
		read(chandata_qt, chandata_acc_x,chandata_acc_y,chandata_acc_z, index, &flag_for_over); 	/* Reading from chandata	*/
		if(FLAG_FOR_DEBUG)
		{
			fp=fopen("debugging.txt","a+"); if(fp==NULL) THROW_ERROR
			fprintf(fp,"Input file succesfully parsed \n");
			fclose(fp);
		}
		flag_for_over=0;
		if(FLAG_ECG)									/*If the ECG sensor is activated then proceed with ECG processing */
		{
			/*printf("...............................\n");*/
				if(FLAG_FOR_DEBUG)
				{				
					fp=fopen("debugging.txt","a+"); if(fp==NULL) THROW_ERROR
					fprintf(fp,"Starting ECG detection routine \n");
					fclose(fp);
				}
				RR_QT=Detect(chandata_qt, &flag_for_over);
				if(FLAG_FOR_DEBUG)
				{				
					fp=fopen("debugging.txt","a+"); if(fp==NULL) THROW_ERROR
					fprintf(fp,"ECG Detect succesfully performed \n");
					fclose(fp);
				}
            			RR=RR_interval();
		/*	printf("Actual RR interval: %.3lf\n", RR_interval() );
			printf("QT pacing: %.3lf\n", RR_QT); */
		}
		if(FLAG_ACC)									/*If the accelerometer sensor is activated then proccced with the Accelerometer processing */
		{
			if(FLAG_FOR_DEBUG)			
			{
				fp=fopen("debugging.txt","a+"); if(fp==NULL) THROW_ERROR
				fprintf(fp,"Starting Accelerometer detection routine \n");
				fclose(fp);
			}
			RR_ACC=accelerometer_processing(chandata_acc_x , chandata_acc_y , chandata_acc_z);
			if(FLAG_FOR_DEBUG)			
			{
				fp=fopen("debugging.txt","a+"); if(fp==NULL) THROW_ERROR
				fprintf(fp,"Accelerometer Detect succesfully performed \n");
				fclose(fp); 
			}
		/*	printf("ACC pacing: %.3lf\n", RR_ACC); */
		}
		if(FLAG_ECG && FLAG_ACC)					/*If blended mode is on, computes the blended pace  */
		{
			RR_BLEND=blending(RR_QT, RR_ACC);
		/*	printf("Blended pacing: %.3lf\n", RR_BLEND);
			printf("...............................\n");*/	
		}
        output_array[0]=RR;
        output_array[1]=RR_BLEND; 
	if(FLAG_FOR_DEBUG)   
	{
		fp=fopen("debugging.txt","a+"); if(fp==NULL) THROW_ERROR
		fprintf(fp,"Blending routine done \n");
		fclose(fp);
	}

}
